@@ -18,7 +18,7 @@ from webob.useragent import UserAgent, parse_search_query
 from webob.etag import AnyETag, NoETag, ETagMatcher, IfRange, NoIfRange
 from webob.headerdict import HeaderDict
 from webob.statusreasons import status_reasons
-from webob.cachecontrol import CacheControl
+from webob.cachecontrol import CacheControl, serialize_cache_control
 from webob.acceptparse import Accept, MIMEAccept, NilAccept, MIMENilAccept, NoAccept
 from webob.byterange import Range, ContentRange
 
@@ -1513,6 +1513,8 @@ class Response(object):
         else:
             return self._app_iter
 
+    ## FIXME: maybe the content-length clearing should only happen if
+    ## self._app_iter is not None?
     def _app_iter__set(self, value):
         if self._body is not None:
             # Undo the automatically-set content-length
@@ -1527,7 +1529,7 @@ class Response(object):
     app_iter = property(_app_iter__get, _app_iter__set, _app_iter__del, doc=_app_iter__get.__doc__)
 
     def set_cookie(self, key, value='', max_age=None,
-                   path='/', domain=None, secure=None):
+                   path='/', domain=None, secure=None, httponly=False):
         """
         Set (add) a cookie for the response
         """
@@ -1538,6 +1540,7 @@ class Response(object):
             ('path', path),
             ('domain', domain),
             ('secure', secure),
+            ('HttpOnly', httponly),
             ]:
             if var_value is not None and var_value is not False:
                 cookies[key][var_name.replace('_', '-')] = str(var_value)
@@ -1647,7 +1650,7 @@ class Response(object):
         if not value:
             value = ""
         if isinstance(value, dict):
-            value = CacheControl(value)
+            value = CacheControl(value, 'response')
         if isinstance(value, unicode):
             value = str(value)
         if isinstance(value, str):
@@ -1662,14 +1665,13 @@ class Response(object):
     def _cache_control__del(self):
         self.cache_control = {}
 
-    def _update_cache_control(self, cache_control_obj):
-        value = str(cache_control_obj)
+    def _update_cache_control(self, prop_dict):
+        value = serialize_cache_control(prop_dict)
         if not value:
             if 'Cache-Control' in self.headers:
                 del self.headers['Cache-Control']
         else:
             self.headers['Cache-Control'] = value
-        cache_control_obj.header_value = value
 
     cache_control = property(_cache_control__get, _cache_control__set, _cache_control__del, doc=_cache_control__get.__doc__)
 
@@ -1694,8 +1696,12 @@ class Response(object):
             cache_control.post_check = 0
             cache_control.pre_check = 0
             self.expires = datetime.utcnow()
+            if 'last-modified' not in self.headers:
+                self.last_modified = datetime.utcnow()
+            self.pragma = 'no-cache'
         else:
             cache_control.max_age = seconds
+            self.expires = datetime.utcnow() + timedelta(seconds=seconds)
         for name, value in kw.items():
             setattr(cache_control, name, value)
 
